@@ -1,4 +1,5 @@
 import math
+import itertools
 
 if __name__ == "__main__":  # to make the tests run without the pytest cli
     import sys
@@ -135,7 +136,12 @@ def test_order():
     assert " ".join(sorted(istr.range(1, 13))) == "1 2 3 4 5 6 7 8 9 10 11 12"
     assert " ".join(sorted(map(istr, range(1, 13)))) == "1 2 3 4 5 6 7 8 9 10 11 12"
 
-
+def test_concat():
+    c = list(istr.concat(((1,2),(3,4))))
+    assert c == istr(['12','34'])
+    c=list(istr.concat(itertools.permutations(range(3),2)))
+    assert c == [istr('01'), istr('02'), istr('10'), istr('12'), istr('20'), istr('21')]
+    
 def test_range():
     assert one_to_twelve == istr.range("1", "13")
     assert one_to_twelve == istr.range(one, thirteen, one)
@@ -266,11 +272,11 @@ def test_format():
             ...
     with istr.format("0"):
         assert istr(" 3 ") == "3"
-    assert istr.default_format() == ""
-    istr.default_format("03")
-    assert istr.default_format() == "03"
+    assert istr.format() == ""
+    istr.format("03")
+    assert istr.format() == "03"
     assert istr("  8 ") == "008"
-    istr.default_format("")
+    istr.format("")
     assert istr(" 8 ") == " 8 "
 
 
@@ -323,6 +329,7 @@ def test_matmul():
     with pytest.raises(TypeError):
         "3" @ five
 
+
 def test_str():
     assert repr(str(five)) == "'5'"
 
@@ -368,7 +375,8 @@ def test_edge_cases():
     assert istr(istr(one)).equals(istr("1"))
     with pytest.raises(TypeError):
         istr()
-
+    rng=istr.range(5)
+    assert rng is istr(rng)
 
 def test_unpacking():
     a = istr("123")
@@ -378,6 +386,70 @@ def test_unpacking():
     assert z.equals(istr(3))
 
 
+def test_repr_mode():
+    hundred = istr(100)
+    assert repr(hundred) == "istr('100')"
+
+    with istr.repr_mode("istr"):
+        hundred = istr(100)
+    assert repr(hundred) == "istr('100')"
+
+    with istr.repr_mode("int"):
+        hundred = istr(100)
+    assert repr(hundred) == "100"
+    with istr.repr_mode("str"):
+        hundred = istr(100)
+    assert repr(hundred) == "'100'"
+    hundred = istr(100)
+    assert repr(hundred) == "istr('100')"
+
+    assert istr.repr_mode() == "istr"
+
+    with pytest.raises(TypeError):
+        istr.repr_mode("no")
+
+
+def test_base():
+    assert istr.base() == 10
+    with istr.base(16):
+        a = istr("7fff")
+        assert a == 32767
+        assert a == "7fff"
+        b = istr("3fff")
+        c = a - b
+        assert c == istr("4000")
+        assert c == 2**14
+    assert istr.base() == 10
+    with istr.base(36):
+        a = istr("PA7")
+        assert a == 32767
+
+    with istr.base(2):
+        a = istr("111")
+        assert a == 7
+        b = a - istr("110")
+        assert repr(b) == "istr('1')"
+
+    with pytest.raises(ValueError):
+        istr.base(1)
+    with pytest.raises(ValueError):
+        istr.base(37)
+
+    with pytest.raises(ValueError):
+        with istr.base(16):
+            istr(-1)
+
+    with istr.base(16):
+        a = istr(15)
+    with istr.base(10):
+        assert a * a == 225
+    with pytest.raises(ValueError):
+        assert a | a # FF can't be converted to base 10
+        
+    
+
+
+
 def test_subclassing():
     class jstr(istr):
         ...
@@ -388,3 +460,4 @@ def test_subclassing():
 
 if __name__ == "__main__":
     pytest.main(["-vv", "-s", "-x", __file__])
+
